import { ApiResponse } from '@api/types';
import { ParkFormValues } from '@components/pages/park-page/types';
import { TurbineTypeFormValues } from '@components/pages/turbine-type-page/types';

import { api } from '../api';
import { WIND_ENDPOINTS } from './constants';
import { Park, ParkTurbine, ParkWithTurbines, TurbineType } from './types';
import { packPark, packParkTurbine, packTurbineTypes } from './utils';

export const getTurbineTypes = () => {
  return api.get<TurbineType[]>(WIND_ENDPOINTS.turbines);
};

export const getTurbineType = (id: string) => {
  const url = `${WIND_ENDPOINTS.turbineType}/${id}`;
  return api.get<TurbineType>(url);
};

export const createTurbineTypes = (
  formValues: Partial<TurbineTypeFormValues>,
) => {
  return api.post<TurbineType>(
    WIND_ENDPOINTS.turbineType,
    packTurbineTypes(formValues),
  );
};

export const editTurbineTypes = (
  formValues: Partial<TurbineTypeFormValues>,
  id: string,
) => {
  const url = `${WIND_ENDPOINTS.turbineType}/${id}`;
  return api.put<TurbineType>(url, packTurbineTypes(formValues));
};

export const deleteTurbineType = (id: number) => {
  const url = `${WIND_ENDPOINTS.turbineType}/${id}`;
  return api.delete(url);
};

export const getParks = () => {
  return api.get<Park[]>(WIND_ENDPOINTS.parks);
};

export const getPark = (id: string) => {
  const url = `${WIND_ENDPOINTS.park}/${id}`;
  return api.get<Park>(url);
};

export const getParkTurbines = (id: string) => {
  const url = `${WIND_ENDPOINTS.parks}/${id}/turbines`;
  return api.get<ParkTurbine[]>(url);
};

export const getParkWithTurbines = async (
  id: string,
): Promise<ApiResponse<ParkWithTurbines>> => {
  const parkURL = `${WIND_ENDPOINTS.park}/${id}`;
  const turbinesURL = `${WIND_ENDPOINTS.parks}/${id}/turbines`;
  const parkPesponse = await api.get<Park>(parkURL);
  const turbinesResponse = await api.get<ParkTurbine[]>(turbinesURL);
  return {
    data: { ...parkPesponse.data, turbines: turbinesResponse.data },
    error: parkPesponse.error || turbinesResponse.error || null,
  };
};

export const createPark = async (formValues: Partial<ParkFormValues>) => {
  const parkPesponse = await api.post<Park>(
    WIND_ENDPOINTS.park,
    packPark(formValues),
  );
  await Promise.all(
    formValues.turbines?.map((t) => {
      return api.post(
        WIND_ENDPOINTS.parkTurbine,
        packParkTurbine(t, parkPesponse.data.id),
      );
    }),
  );
  return getParkWithTurbines(String(parkPesponse.data.id));
};

export const updatePark = async (
  formValues: Partial<ParkFormValues>,
  id: string,
) => {
  const parkPesponse = await api.put<Park>(
    `${WIND_ENDPOINTS.park}/${id}`,
    packPark(formValues),
  );
  await Promise.all(
    formValues.turbines?.map((t) => {
      if (t.new) {
        return api.post(
          WIND_ENDPOINTS.parkTurbine,
          packParkTurbine(t, parkPesponse.data.id),
        );
      }
      if (t.delete) {
        return api.delete(
          `${WIND_ENDPOINTS.parkTurbine}/${parkPesponse.data.id}/${t.id}`,
        );
      }
      return api.put(
        `${WIND_ENDPOINTS.parkTurbine}/${parkPesponse.data.id}/${t.id}`,
        packParkTurbine(t, parkPesponse.data.id),
      );
    }),
  );
  return getParkWithTurbines(id);
};

export const deletePark = (id: number) => {
  const url = `${WIND_ENDPOINTS.park}/${id}`;
  return api.delete(url);
};
