import {
  createPark,
  getParkWithTurbines,
  ParkWithTurbines,
  updatePark,
} from '@api/wind';
import {
  Button,
  Dialog,
  Heading,
  NumberField,
  TextInput,
} from '@components/ui';
import { ParkTurbines } from '@components/ux';
import { Controller, useForm } from '@utils/form';
import { ROUTES, useRoute } from '@utils/route';
import React, { useEffect, useState } from 'react';
import { useNavigate, useParams } from 'react-router-dom';

import styles from './styles.module.scss';
import { ParkFormValues } from './types';
import { unpackPark } from './utils';

export function ParkPage() {
  const [park, setPark] = useState<ParkWithTurbines>(null);
  const [pending, setPending] = useState<boolean>(false);
  const [error, setError] = useState<string>(null);
  const params = useParams();
  const navigate = useNavigate();
  const route = useRoute();

  const { register, control, getValues, reset } = useForm<ParkFormValues>({});

  const { id } = params;
  const isEdit = id !== 'new';
  const heading = isEdit ? 'Edit' : 'Create new';

  const fetchPark = async () => {
    const response = await getParkWithTurbines(id);
    setPark(response.data);
    reset(unpackPark(response.data));
  };

  useEffect(() => {
    if (!isEdit) {
      return;
    }
    fetchPark();
  }, [id]);

  const handleFormSubmit = async (event: React.FormEvent) => {
    event.preventDefault();
    setPending(true);
    if (isEdit) {
      const { data, error } = await updatePark(getValues(), id);
      if (error) {
        setError('Something went wrong');
        return;
      }
      setPark(data);
      reset(unpackPark(data));
    } else {
      const { data, error } = await createPark(getValues());
      if (error) {
        setError('Something went wrong');
        return;
      }
      navigate(ROUTES.park.path.replace(':id', String(data.id)));
    }
    setPending(false);
  };

  const handleReset = () => {
    if (isEdit) {
      reset(unpackPark(park));
    } else {
      reset({});
    }
  };

  return (
    <div className={styles.page} onSubmit={handleFormSubmit}>
      <Heading tag="h1">{route.title}</Heading>
      <form className={styles.form}>
        <header>
          <Heading tag="h3">{heading}</Heading>
        </header>
        <TextInput {...register('name')} label={{ text: 'Name' }} />
        <div className={styles.inputBox}>
          <Controller
            {...control('centerLatitude')}
            render={(props) => (
              <NumberField label={{ text: 'Center Latitude' }} {...props} />
            )}
          />
          <Controller
            {...control('centerLongitude')}
            render={(props) => (
              <NumberField label={{ text: 'Center Longitude' }} {...props} />
            )}
          />
        </div>
        <div className={styles.buttonBox}>
          <Button variant="secondary" onClick={handleReset}>
            Reset
          </Button>
          <Button type="submit" pending={pending}>
            Submit
          </Button>
        </div>
      </form>
      <Controller
        {...control('turbines')}
        render={(props) => (
          <ParkTurbines savedTurbines={park?.turbines ?? []} {...props} />
        )}
      />
      <Dialog
        open={Boolean(error)}
        heading="Error"
        message="Something went wrong"
        onClose={() => setError(null)}
      >
        <Button onClick={() => setError(null)}>Ok</Button>
      </Dialog>
    </div>
  );
}
