import ArrowDownIcon from '@public/images/svg/arrow-down.svg';
import { useMissClick } from '@utils/miss-click';
import clsx from 'clsx';
import React, {
  ForwardedRef,
  forwardRef,
  useImperativeHandle,
  useRef,
  useState,
} from 'react';

import { Menu } from '../menu';
import { Popover } from '../popover';
import { TextInput } from '../text-input';
import styles from './styles.module.scss';
import { AutocompleteProps } from './types';

function AutocompleteInner<T>(
  {
    options,
    value,
    getOptionKey,
    getOptionLabel,
    onChange,
    scale = 'm',
    label = {},
    name,
    id,
  }: Omit<AutocompleteProps<T>, 'ref'>,
  ref: ForwardedRef<HTMLDivElement>,
) {
  const autocompleteRef = useRef<HTMLDivElement | null>(null);
  const menuRef = useRef<HTMLUListElement | null>(null);
  const inputWrapperRef = useRef<HTMLDivElement | null>(null);
  const [menuVisible, setMenuVisible] = useState<boolean>(false);
  const [text, setText] = useState<string>('');

  useImperativeHandle(ref, () => autocompleteRef.current, []);

  useMissClick({
    callback: () => setMenuVisible(false),
    enabled: menuVisible,
    whitelist: [autocompleteRef, menuRef],
  });

  const autocompleteClassName = clsx(styles.autocomplete, styles[scale], {
    [styles.menuVisible]: menuVisible,
  });

  const filteredOptions = options.filter((option) => {
    const label = getOptionLabel(option).toLocaleLowerCase();
    const raw = text.trim().toLocaleLowerCase();
    return label.includes(raw);
  });

  const handleInputClick = () => {
    setMenuVisible(!menuVisible);
  };

  const handleMenuSelect = (option: T) => {
    setMenuVisible(false);
    onChange?.(option);
    setText('');
  };

  const handleInputChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const { value } = event.target;
    setText(value);
    const option = options.find((option) => {
      const label = getOptionLabel(option).toLocaleLowerCase();
      const raw = value.toLocaleLowerCase();
      return label === raw;
    });
    onChange?.(option ?? null);
  };

  return (
    <div className={autocompleteClassName} ref={autocompleteRef}>
      <TextInput
        value={value ? getOptionLabel(value) : text}
        onClick={handleInputClick}
        scale={scale}
        label={label}
        name={name}
        id={id}
        wrapper={{ ref: inputWrapperRef }}
        onChange={handleInputChange}
        rightNode={
          <div className={styles.iconBox}>
            <ArrowDownIcon className={styles.icon} />
          </div>
        }
      />
      <Popover
        visible={menuVisible}
        anchorRef={autocompleteRef}
        position="bottom"
        horizontalAlign="stretch"
        flip
        element={
          <div className={styles.menuWrapper}>
            <Menu
              options={filteredOptions}
              selected={value}
              getOptionKey={getOptionKey}
              getOptionLabel={getOptionLabel}
              onSelect={handleMenuSelect}
              ref={menuRef}
            />
          </div>
        }
      />
    </div>
  );
}

export const Autocomplete = forwardRef(AutocompleteInner) as <T>(
  props: AutocompleteProps<T>,
) => ReturnType<typeof AutocompleteInner>;
