import React, { useMemo, useState } from 'react';

import { DataGridHeader, DataGridRow } from './components';
import { DataGridProps } from './types';

export function DataGrid<T>({
  items,
  columns,
  getItemKey,
  selectedItems,
  onItemsSelect,
  multiselect,
  className,
  ...props
}: DataGridProps<T>) {
  const [allItemsSelected, setAllItemsSelected] = useState<boolean>(false);

  const columnsTemplate = useMemo(() => {
    const main = columns.map((c) => `${c.width ?? 1}fr`).join(' ');
    return `auto ${main}`;
  }, []);

  const selectedItemsMap = useMemo(() => {
    const map: Record<string, T> = {};
    for (let i = 0; i < selectedItems.length; i += 1) {
      const item = selectedItems[i];
      map[String(getItemKey(item))] = item;
    }
    return map;
  }, [selectedItems]);

  const handleSelectAllItems = () => {
    if (!multiselect) {
      return;
    }
    onItemsSelect?.(allItemsSelected ? [] : [...items]);
    setAllItemsSelected(!allItemsSelected);
  };

  const handleItemSelect = (key: string, item: T) => {
    const selected = Boolean(selectedItemsMap[key]);
    if (!multiselect) {
      onItemsSelect?.(selected ? [] : [item]);
      return;
    }
    onItemsSelect?.(
      selected
        ? selectedItems.filter((i) => key !== getItemKey(i))
        : [...selectedItems, item],
    );
    setAllItemsSelected(false);
  };

  return (
    <div className={className} {...props}>
      <DataGridHeader
        columns={columns}
        allItemsSelected={allItemsSelected}
        onSelectAllItems={handleSelectAllItems}
        columnsTemplate={columnsTemplate}
      />
      {items.map((item) => {
        const key = String(getItemKey(item));
        return (
          <DataGridRow
            object={item}
            columns={columns}
            selected={Boolean(selectedItemsMap[key])}
            onSelect={() => handleItemSelect(key, item)}
            key={getItemKey(item)}
            columnsTemplate={columnsTemplate}
          />
        );
      })}
    </div>
  );
}
