import CalendarIcon from '@public/images/svg/calendar.svg';
import { useMissClick } from '@utils/miss-click';
import React, { useEffect, useMemo, useRef, useState } from 'react';

import { Calendar } from '../calendar';
import { IconButton } from '../icon-button';
import { Popover } from '../popover';
import { TextInput } from '../text-input';
import styles from './styles.module.scss';
import { DateInputProps } from './types';
import { dirtyDateToValue, inputToDirtyDate, valueToDirtyDate } from './utils';

export function DateInput({
  value,
  onChange,
  rightNode,
  scale,
  min,
  max,
  ...props
}: DateInputProps) {
  const wrapperRef = useRef<HTMLDivElement | null>(null);
  const inputWrapperRef = useRef<HTMLDivElement | null>(null);
  const calendarWrapperRef = useRef<HTMLDivElement | null>(null);
  const [calendarVisible, setCalendarVisible] = useState<boolean>(false);
  const [dirtyDate, setDirtyDate] = useState<string>('');

  const minDate = useMemo(() => {
    return min ? new Date(min) : null;
  }, [min]);

  const maxDate = useMemo(() => {
    return max ? new Date(max) : null;
  }, [max]);

  useEffect(() => {
    setDirtyDate(valueToDirtyDate(value));
  }, [value]);

  useMissClick({
    callback: () => setCalendarVisible(false),
    enabled: calendarVisible,
    whitelist: [wrapperRef, calendarWrapperRef],
  });

  const handleCalendarButtonClick = () => {
    setCalendarVisible(!calendarVisible);
  };

  const handleCalendarButtonMouseDown = (event: React.MouseEvent) => {
    event.preventDefault();
  };

  const handleCalendarButtonMouseUp = (event: React.MouseEvent) => {
    event.preventDefault();
  };

  const handleInputChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const newDirtyDate = inputToDirtyDate(event.target.value);
    if (newDirtyDate.length === 10) {
      const newValue = dirtyDateToValue(newDirtyDate);
      const date = new Date(newValue);
      if (
        !isNaN(date.getDate()) &&
        (!minDate || date >= minDate) &&
        (!maxDate || date <= maxDate)
      ) {
        onChange(newValue);
      } else {
        onChange('');
      }
    }
    setDirtyDate(newDirtyDate);
  };

  const handleCalendarChange = (newValue: string) => {
    onChange(newValue);
    setCalendarVisible(false);
  };

  return (
    <div className={styles.wrapper} ref={wrapperRef}>
      <TextInput
        {...props}
        scale={scale}
        value={dirtyDate}
        onChange={handleInputChange}
        wrapper={{ ref: inputWrapperRef }}
        rightNode={
          <>
            <IconButton
              scale={scale}
              onClick={handleCalendarButtonClick}
              onMouseDown={handleCalendarButtonMouseDown}
              onMouseUp={handleCalendarButtonMouseUp}
            >
              <CalendarIcon />
            </IconButton>
            {rightNode}
          </>
        }
      />
      <Popover
        visible={calendarVisible}
        anchorRef={wrapperRef}
        position="bottom"
        horizontalAlign="center"
        flip
        element={
          <div className={styles.calendarWrapper} ref={calendarWrapperRef}>
            <Calendar
              value={value}
              onChange={handleCalendarChange}
              className={styles.calendar}
              min={minDate}
              max={maxDate}
            />
          </div>
        }
      />
    </div>
  );
}
