import math
import os.path
import sys
import uuid
from pathlib import Path
from typing import Set

from floris import FlorisModel
from floris.flow_visualization import visualize_cut_plane
from floris.layout_visualization import plot_turbine_labels
import floris.layout_visualization as layoutviz

import matplotlib.pyplot as plt

_wind_direction_to_val: dict[str, float] = {
    "N": 0.0,
    "E": 90.0,
    "S": 180.0,
    "W": 270.0,
}


def _convert_winds_list_direction_definitions(
        wind_directions: list[str],
) -> list[float]:
    return list(map(lambda x: _convert_wind_direction_definition(x.upper()), wind_directions))


def _convert_wind_direction_definition(
        wind_direction: str
) -> float:
    if not _check_wind_direction_definition(wind_direction):
        raise ValueError(f"Wind direction {wind_direction} is not valid")

    return sum(map(lambda x: _wind_direction_to_val[x], wind_direction)) / len(wind_direction)


def _check_wind_direction_definition(wind_direction: str) -> bool:
    if len(set(wind_direction) | set(_wind_direction_to_val)) > len(_wind_direction_to_val):
        return False
    return True


class FlorisULSTU(FlorisModel):
    def __init__(
            self,
            url_yaml: Path = Path(__file__).parent / "gch.yaml"
    ):
        super().__init__(url_yaml)

    def set(self, **kwargs):
        if ("wind_directions" in kwargs) and (type(kwargs["wind_directions"][0]) is str):
            kwargs["wind_directions"] = _convert_winds_list_direction_definitions(kwargs["wind_directions"])

        super().set(**kwargs)

    def visualization(
            self,
            interest_plots: Set[str]
    ):
        dct_functions = {
            "horizontal_plane": self._horizontal_plane_visualization,
            "vertical_plane": self._vertical_plane_visualization,
        }

        res = {}

        for param in interest_plots:
            if param not in dct_functions:
                res[param] = "Not found"
                continue
            res[param] = dct_functions[param]()

        return res

    def _horizontal_plane_visualization(self):
        return self._plane_visualization(self.calculate_horizontal_plane, {"height": 90.0})

    def _vertical_plane_visualization(self):
        return self._plane_visualization(self.calculate_y_plane,
                                         {"crossstream_dist": 0.0,
                                          "x_resolution": 200,
                                          "z_resolution": 100})

    def _plane_visualization(self, plane_func, func_params: dict[str, float] = {}):
        wind_dirs = self.core.flow_field.wind_directions
        wind_speeds = self.core.flow_field.wind_speeds

        experiment_counts = len(wind_dirs)
        near_square = math.ceil(math.sqrt(experiment_counts))
        fig, ax_arr = plt.subplots(near_square, near_square, figsize=(15, 8))
        ax_arr = ax_arr.flatten()

        for experiment_count in range(experiment_counts):
            self.reset_operation()
            self.set(wind_speeds=[wind_speeds[experiment_count]], wind_directions=[wind_dirs[experiment_count]],
                     turbulence_intensities=[0.05])
            ax = ax_arr[experiment_count]
            plane = plane_func(**func_params)
            visualize_cut_plane(
                plane,
                ax=ax,
                title=f"Wind flow day {experiment_count + 1}",
            )

        filename = str(uuid.uuid4()) + ".png"
        plt.savefig(Path(__file__).parent.parent.parent / f"public/floris/{filename}")
        plt.close()

        self.set(wind_speeds=wind_speeds, wind_directions=wind_dirs, turbulence_intensities=[0.05] * len(wind_dirs))

        return filename


    def _cross_visualization(self, plane_func, func_params: dict[str, float] = {}):
        wind_dirs = self.core.flow_field.wind_directions
        wind_speeds = self.core.flow_field.wind_speeds


        experiment_counts = len(wind_dirs)
        near_square = math.ceil(math.sqrt(experiment_counts))
        fig, ax_arr = plt.subplots(near_square, near_square, figsize=(15, 8))
        ax_arr = ax_arr.flatten()

        for experiment_count in range(experiment_counts):
            self.reset_operation()
            self.set(wind_speeds=[wind_speeds[experiment_count]], wind_directions=[wind_dirs[experiment_count]],
                     turbulence_intensities=[0.05])
            ax = ax_arr[experiment_count]
            plane = plane_func(**func_params)
            visualize_cut_plane(
                plane,
                ax=ax,
                title=f"Wind flow day {experiment_count + 1}",
            )

        filename = str(uuid.uuid4()) + ".png"
        plt.savefig(Path(__file__).parent.parent.parent / f"public/floris/{filename}")
        plt.close()

        self.set(wind_speeds=wind_speeds, wind_directions=wind_dirs, turbulence_intensities=[0.05] * len(wind_dirs))

        return filename