from typing import List, Type

from sqlalchemy import select

from sqlalchemy.orm import Session
from .models import WindTurbineType, WindPark, WindParkTurbine
from .schemas import WindTurbineTypeCreate, WindParkCreate, WindParkTurbineCreate, WindParkResponse, \
    WindTurbineResponse, WindParkTurbineResponse, WindTurbineWithParkDetailsResponse
from data.database import SessionLocal
from data.models import Weather
from data.schemas import SWeatherInfo


class WeatherRepository:
    @classmethod
    def get_all(cls):
        with SessionLocal() as session:
            query = (
                select(Weather)
            )
            res = session.execute(query)
            weather_models = res.scalars().all()
            return [SWeatherInfo.model_validate(weather, from_attributes=True) for weather in weather_models]

    @classmethod
    def get_by_id(cls, id):
        with SessionLocal() as session:
            query = (
                select(Weather).where(Weather.id == id)
            )
            res = session.execute(query)
            weather_model = res.scalars().first()
            return SWeatherInfo.model_validate(weather_model, from_attributes=True)



class WindTurbineTypeRepository:
    @staticmethod
    def create(db: Session, turbine_type: WindTurbineTypeCreate):
        db_turbine_type = WindTurbineType(**turbine_type.dict())
        db.add(db_turbine_type)
        db.commit()
        db.refresh(db_turbine_type)
        return db_turbine_type

    @staticmethod
    def get(db: Session, turbine_type_id: int):
        return db.query(WindTurbineType).filter(WindTurbineType.Id == turbine_type_id).first()

    @staticmethod
    def update(db: Session, turbine_type_id: int, turbine_type: WindTurbineTypeCreate):
        db_turbine_type = db.query(WindTurbineType).filter(WindTurbineType.Id == turbine_type_id).first()
        if db_turbine_type:
            for key, value in turbine_type.dict().items():
                setattr(db_turbine_type, key, value)
            db.commit()
            db.refresh(db_turbine_type)
            return db_turbine_type
        return None

    @staticmethod
    def delete(db: Session, turbine_type_id: int):
        db_turbine_type = db.query(WindTurbineType).filter(WindTurbineType.Id == turbine_type_id).first()
        if db_turbine_type:
            db.delete(db_turbine_type)
            db.commit()
            return True
        return False

    @staticmethod
    def get_all_turbines(db: Session) -> list[Type[WindTurbineType]]:
        return db.query(WindTurbineType).all()


class WindParkRepository:
    @staticmethod
    def create(db: Session, park: WindParkCreate):
        db_park = WindPark(**park.dict())
        db.add(db_park)
        db.commit()
        db.refresh(db_park)
        return db_park

    @staticmethod
    def get(db: Session, park_id: int):
        return db.query(WindPark).filter(WindPark.Id == park_id).first()

    @staticmethod
    def update(db: Session, park_id: int, park: WindParkCreate):
        db_park = db.query(WindPark).filter(WindPark.Id == park_id).first()
        if db_park:
            for key, value in park.dict().items():
                setattr(db_park, key, value)
            db.commit()
            db.refresh(db_park)
            return db_park
        return None

    @staticmethod
    def delete(db: Session, park_id: int):
        db_park = db.query(WindPark).filter(WindPark.Id == park_id).first()
        if db_park:
            db.delete(db_park)
            db.commit()
            return True
        return False

    @staticmethod
    def get_all_parks(db: Session) -> list[WindParkResponse]:
        return db.query(WindPark).all()

    @staticmethod
    def get_turbines_by_park_id(park_id: int, db: Session) -> list[WindTurbineWithParkDetailsResponse]:
        turbines = (
            db.query(
                WindTurbineType.Id.label("Id"),
                WindTurbineType.Name.label("Name"),
                WindTurbineType.Height.label("Height"),
                WindTurbineType.BladeLength.label("BladeLength"),
                WindParkTurbine.x_offset.label("x_offset"),
                WindParkTurbine.y_offset.label("y_offset"),
                WindParkTurbine.angle.label("angle"),
                WindParkTurbine.comment.label("comment"),
            )
            .join(WindParkTurbine, WindTurbineType.Id == WindParkTurbine.turbine_id)
            .filter(WindParkTurbine.wind_park_id == park_id)
            .all()
        )

        if not turbines:
            return []  # Возвращаем пустой список, если не найдено

        return [WindTurbineWithParkDetailsResponse(**turbine._asdict()) for turbine in turbines]


class WindParkTurbineRepository:
    @staticmethod
    def create(db: Session, park_turbine: WindParkTurbineCreate):
        db_park_turbine = WindParkTurbine(**park_turbine.dict())
        db.add(db_park_turbine)
        db.commit()
        db.refresh(db_park_turbine)
        return db_park_turbine

    @staticmethod
    def get(db: Session, park_id: int, turbine_id: int):
        return db.query(WindParkTurbine).filter(WindParkTurbine.wind_park_id == park_id, WindParkTurbine.turbine_id == turbine_id).first()

    @staticmethod
    def update(db: Session, park_id: int, turbine_id: int, park_turbine: WindParkTurbineCreate):
        db_park_turbine = db.query(WindParkTurbine).filter(WindParkTurbine.wind_park_id == park_id, WindParkTurbine.turbine_id == turbine_id).first()
        if db_park_turbine:
            for key, value in park_turbine.dict().items():
                setattr(db_park_turbine, key, value)
            db.commit()
            db.refresh(db_park_turbine)
            return db_park_turbine
        return None

    @staticmethod
    def delete(db: Session, park_id: int, turbine_id: int):
        db_park_turbine = db.query(WindParkTurbine).filter(WindParkTurbine.wind_park_id == park_id, WindParkTurbine.turbine_id == turbine_id).first()
        if db_park_turbine:
            db.delete(db_park_turbine)
            db.commit()
            return True
        return False
