import sys
from http import HTTPStatus
from pathlib import Path
from typing import Annotated

from fastapi import APIRouter, HTTPException, Depends
from fastapi.responses import FileResponse

from data.repository import WeatherRepository
from data.schemas import SFlorisInputParams, SFlorisOutputData, SWeatherInfo
import numpy as np


sys.path.append(str(Path(__file__).parent.parent.parent))
from floris_module.src import FlorisULSTU
from floris_module.src.OpenMeteoClient import OpenMeteoClient


FLORIS_IMAGES_PATH = Path(__file__).parent.parent.parent / "public" / "floris"

router = APIRouter(
    prefix="/api/floris",
    tags=["Floris Api"],
)


@router.get("/get_windmill_data")
async def get_windmill_data(
    data: Annotated[SFlorisInputParams, Depends()]
):
    if len(data.layout_x) != len(data.layout_y) and len(data.layout_x) != len(data.yaw_angle):
        raise HTTPException(
            status_code=HTTPStatus.BAD_REQUEST,
            detail="Length of layout x and y and yaw_angle must be the same",
        )


    fmodel = FlorisULSTU()

    client = OpenMeteoClient()

    climate_info = client.get_weather_info(data.date_start, data.date_end)
    wind_speeds = climate_info[0]
    wind_directions = climate_info[1]

    fmodel.set(
        layout_x=data.layout_x,
        layout_y=data.layout_y,
        wind_directions=wind_directions,
        wind_speeds=wind_speeds,
        turbulence_intensities=[0.1] * len(wind_directions)
    )

    yaw_angles = np.zeros((len(wind_directions), len(data.layout_x)))
    for i in range(len(data.layout_x)):
        yaw_angles[:, i] = data.yaw_angle[i]

    fmodel.set(
        yaw_angles=yaw_angles,
    )

    fmodel.run()

    res = fmodel.get_turbine_powers().tolist()
    file_names = fmodel.visualization(data.plots)

    return SFlorisOutputData(
        file_name=file_names,
        data=res
    )


@router.get("/get_images")
async def get_images():
    try:
        images = [file.name for file in FLORIS_IMAGES_PATH.iterdir() if file.is_file()]
        return {"images": images}
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=str(e)
        )


@router.get("/download_image/{image_name}")
async def download_image(
        image_name: str
):
    image_path = FLORIS_IMAGES_PATH / image_name
    if not image_path.exists() or not image_path.is_file():
        raise HTTPException(status_code=404, detail="Image not found")
    return FileResponse(image_path, media_type="image/jpeg", filename=image_name)



