from fastapi import APIRouter, HTTPException, Depends
from sqlalchemy.orm import Session

from data.schemas import WindTurbineTypeCreate, WindTurbineTypeResponse, WindParkCreate, WindParkResponse, \
    WindParkTurbineCreate, WindParkTurbineResponse, WindTurbineResponse, WindTurbineWithParkDetailsResponse
from data.database import get_db
from data.repository import WindTurbineTypeRepository, WindParkRepository, WindParkTurbineRepository

router = APIRouter(
    prefix="/api/wind",
    tags=["Wind API"],
)


# Wind Turbine Type CRUD
@router.post("/turbine_type/", response_model=WindTurbineTypeResponse)
async def create_turbine_type(turbine_type: WindTurbineTypeCreate, db: Session = Depends(get_db)):
    return WindTurbineTypeRepository.create(db=db, turbine_type=turbine_type)


@router.get("/turbine_type/{turbine_type_id}", response_model=WindTurbineTypeResponse)
async def read_turbine_type(turbine_type_id: int, db: Session = Depends(get_db)):
    turbine_type = WindTurbineTypeRepository.get(db=db, turbine_type_id=turbine_type_id)
    if turbine_type is None:
        raise HTTPException(status_code=404, detail="Turbine Type not found")
    return turbine_type


@router.put("/turbine_type/{turbine_type_id}", response_model=WindTurbineTypeResponse)
async def update_turbine_type(turbine_type_id: int, turbine_type: WindTurbineTypeCreate, db: Session = Depends(get_db)):
    updated_turbine_type = WindTurbineTypeRepository.update(db=db, turbine_type_id=turbine_type_id,
                                                            turbine_type=turbine_type)
    if updated_turbine_type is None:
        raise HTTPException(status_code=404, detail="Turbine Type not found")
    return updated_turbine_type


@router.delete("/turbine_type/{turbine_type_id}", status_code=204)
async def delete_turbine_type(turbine_type_id: int, db: Session = Depends(get_db)):
    result = WindTurbineTypeRepository.delete(db=db, turbine_type_id=turbine_type_id)
    if not result:
        raise HTTPException(status_code=404, detail="Turbine Type not found")


# Wind Park CRUD
@router.post("/park/", response_model=WindParkResponse)
async def create_park(park: WindParkCreate, db: Session = Depends(get_db)):
    return WindParkRepository.create(db=db, park=park)


@router.get("/park/{park_id}", response_model=WindParkResponse)
async def read_park(park_id: int, db: Session = Depends(get_db)):
    park = WindParkRepository.get(db=db, park_id=park_id)
    if park is None:
        raise HTTPException(status_code=404, detail="Park not found")
    return park


@router.put("/park/{park_id}", response_model=WindParkResponse)
async def update_park(park_id: int, park: WindParkCreate, db: Session = Depends(get_db)):
    updated_park = WindParkRepository.update(db=db, park_id=park_id, park=park)
    if updated_park is None:
        raise HTTPException(status_code=404, detail="Park not found")
    return updated_park


@router.delete("/park/{park_id}", status_code=204)
async def delete_park(park_id: int, db: Session = Depends(get_db)):
    result = WindParkRepository.delete(db=db, park_id=park_id)
    if not result:
        raise HTTPException(status_code=404, detail="Park not found")


# Wind Park Turbine CRUD
@router.post("/park_turbine/", response_model=WindParkTurbineResponse)
async def create_park_turbine(park_turbine: WindParkTurbineCreate, db: Session = Depends(get_db)):
    return WindParkTurbineRepository.create(db=db, park_turbine=park_turbine)


@router.get("/park_turbine/{park_id}/{turbine_id}", response_model=WindParkTurbineResponse)
async def read_park_turbine(park_id: int, turbine_id: int, db: Session = Depends(get_db)):
    park_turbine = WindParkTurbineRepository.get(db=db, park_id=park_id, turbine_id=turbine_id)
    if park_turbine is None:
        raise HTTPException(status_code=404, detail="Park Turbine not found")
    return park_turbine


@router.put("/park_turbine/{park_id}/{turbine_id}", response_model=WindParkTurbineResponse)
async def update_park_turbine(park_id: int, turbine_id: int, park_turbine: WindParkTurbineCreate, db: Session = Depends(get_db)):
    updated_park_turbine = WindParkTurbineRepository.update(db=db, park_id=park_id, turbine_id=turbine_id,
                                                            park_turbine=park_turbine)
    if updated_park_turbine is None:
        raise HTTPException(status_code=404, detail="Park Turbine not found")
    return updated_park_turbine


@router.delete("/park_turbine/{park_id}/{turbine_id}", status_code=204)
async def delete_park_turbine(park_id: int, turbine_id: int, db: Session = Depends(get_db)):
    result = WindParkTurbineRepository.delete(db=db, park_id=park_id, turbine_id=turbine_id)
    if not result:
        raise HTTPException(status_code=404, detail="Park Turbine not found")


@router.get("/parks/", response_model=list[WindParkResponse])
async def get_all_parks(db: Session = Depends(get_db)):
    """Получить все ветропарки"""
    return WindParkRepository.get_all_parks(db)


@router.get("/parks/{park_id}/turbines/", response_model=list[WindTurbineWithParkDetailsResponse])
async def get_turbines_by_park_id(park_id: int, db: Session = Depends(get_db)):
    """Получить все турбины в ветропарке по ID"""
    turbines = WindParkRepository.get_turbines_by_park_id(park_id, db)

    return turbines


@router.get("/turbines/", response_model=list[WindTurbineResponse])
async def get_all_turbines(db: Session = Depends(get_db)):
    """Получить все турбины"""
    return WindTurbineTypeRepository.get_all_turbines(db)
